"use strict";(self.webpackChunkblrec=self.webpackChunkblrec||[]).push([[103],{5103:(k,h,e)=>{e.r(h),e.d(h,{AboutModule:()=>c});var m=e(6895),f=e(6582),t=e(4650),A=e(4670),d=e(529),v=e(7930);class s{constructor(n,i){this.http=n,this.url=i}getLatestVerisonString(){const n=this.url.makeApiUrl("/api/v1/update/version/latest");return this.http.get(n)}}s.\u0275fac=function(n){return new(n||s)(t.LFG(d.eN),t.LFG(v.i))},s.\u0275prov=t.Yz7({token:s,factory:s.\u0275fac,providedIn:"root"});class u{constructor(n){this.latestVesion$=n.getLatestVerisonString()}}u.\u0275fac=function(n){return new(n||u)(t.Y36(s))},u.\u0275cmp=t.Xpm({type:u,selectors:[["app-info-list"]],inputs:{appInfo:"appInfo"},decls:42,vars:4,consts:[[1,"info-list"],[1,"info-item"],[1,"label"],[1,"desc"],["href","https://github.com/acgnhiki/blrec","target","_blank"],["href","https://github.com/acgnhiki/blrec/issues","target","_blank"],["href","https://choosealicense.com/licenses/gpl-3.0","target","_blank"],["href","mailto:acgnhiki@outlook.com","target","_blank"],["href","https://afdian.net/@acgnhiki","target","_blank"]],template:function(n,i){1&n&&(t.TgZ(0,"ul",0)(1,"li",1)(2,"span",2),t._uU(3,"\u5f53\u524d\u7248\u672c"),t.qZA(),t.TgZ(4,"span",3),t._uU(5),t.qZA()(),t.TgZ(6,"li",1)(7,"span",2),t._uU(8,"\u6700\u65b0\u7248\u672c"),t.qZA(),t.TgZ(9,"span",3),t._uU(10),t.ALo(11,"async"),t.qZA()(),t.TgZ(12,"li",1)(13,"span",2),t._uU(14,"\u9879\u76ee\u4e3b\u9875"),t.qZA(),t.TgZ(15,"span",3)(16,"a",4),t._uU(17,"https://github.com/acgnhiki/blrec"),t.qZA()()(),t.TgZ(18,"li",1)(19,"span",2),t._uU(20,"\u95ee\u9898\u53cd\u9988"),t.qZA(),t.TgZ(21,"span",3)(22,"a",5),t._uU(23,"https://github.com/acgnhiki/blrec/issues"),t.qZA()()(),t.TgZ(24,"li",1)(25,"span",2),t._uU(26,"\u8bb8\u53ef\u534f\u8bae"),t.qZA(),t.TgZ(27,"span",3)(28,"a",6),t._uU(29,"GNU GPLv3"),t.qZA()()(),t.TgZ(30,"li",1)(31,"span",2),t._uU(32,"\u8054\u7cfb\u65b9\u5f0f"),t.qZA(),t.TgZ(33,"span",3)(34,"a",7),t._uU(35,"acgnhiki@outlook.com"),t.qZA()()(),t.TgZ(36,"li",1)(37,"span",2),t._uU(38,"\u6295\u5582\u8d5e\u52a9"),t.qZA(),t.TgZ(39,"span",3)(40,"a",8),t._uU(41,"https://afdian.net/@acgnhiki"),t.qZA()()()()),2&n&&(t.xp6(5),t.Oqu(i.appInfo.version),t.xp6(5),t.Oqu(t.lcZ(11,2,i.latestVesion$)))},dependencies:[m.Ov],styles:['@charset "UTF-8";.info-list[_ngcontent-%COMP%]   .info-item[_ngcontent-%COMP%]{display:flex;flex-wrap:wrap;align-items:center;margin:0;padding:1em 2em;border-top:1px solid rgba(0,0,0,.06)}.info-list[_ngcontent-%COMP%]   .info-item[_ngcontent-%COMP%]:first-child{border-top:none}.info-list[_ngcontent-%COMP%]   .info-item[_ngcontent-%COMP%]   .label[_ngcontent-%COMP%]:after{content:"\\ff1a"}.info-list[_ngcontent-%COMP%]{margin:0;padding:0;list-style:none}'],changeDetection:0});class l{constructor(n,i){this.changeDetector=n,this.route=i}ngOnInit(){this.route.data.subscribe(n=>{this.appInfo=n.appInfo,this.changeDetector.markForCheck()})}}l.\u0275fac=function(n){return new(n||l)(t.Y36(t.sBO),t.Y36(f.gz))},l.\u0275cmp=t.Xpm({type:l,selectors:[["app-about"]],decls:4,vars:1,consts:[[1,"inner-content"],[1,"about-page"],[3,"appInfo"]],template:function(n,i){1&n&&(t.TgZ(0,"div",0)(1,"div",1)(2,"app-page-section"),t._UZ(3,"app-info-list",2),t.qZA()()()),2&n&&(t.xp6(3),t.Q6J("appInfo",i.appInfo))},dependencies:[A.g,u],styles:[".inner-content[_ngcontent-%COMP%]{height:100%;width:100%;position:relative;display:block;margin:0;padding:1rem;background:#f1f1f1;overflow:auto}.inner-content[_ngcontent-%COMP%]   .about-page[_ngcontent-%COMP%]{max-width:680px;margin:0 auto}"]});var Z=e(262),b=e(4007),C=e(324),U=e(387);class r{constructor(n,i){this.http=n,this.url=i}getAppInfo(){const n=this.url.makeApiUrl("/api/v1/app/info");return this.http.get(n)}getAppStatus(){const n=this.url.makeApiUrl("/api/v1/app/status");return this.http.get(n)}restartApp(){const n=this.url.makeApiUrl("/api/v1/app/restart");return this.http.post(n,null)}exitApp(){const n=this.url.makeApiUrl("/api/v1/app/exit");return this.http.post(n,null)}}r.\u0275fac=function(n){return new(n||r)(t.LFG(d.eN),t.LFG(v.i))},r.\u0275prov=t.Yz7({token:r,factory:r.\u0275fac,providedIn:"root"});class a{constructor(n,i,g){this.logger=n,this.notification=i,this.appService=g}resolve(n,i){return this.appService.getAppInfo().pipe((0,b.X)(3,300),(0,Z.K)(g=>{throw this.logger.error("Failed to get app info:",g),this.notification.error("\u83b7\u53d6\u540e\u7aef\u5e94\u7528\u4fe1\u606f\u51fa\u9519",g.message,{nzDuration:0}),g}))}}a.\u0275fac=function(n){return new(n||a)(t.LFG(C.Kf),t.LFG(U.zb),t.LFG(r))},a.\u0275prov=t.Yz7({token:a,factory:a.\u0275fac});const F=[{path:"",component:l,resolve:{appInfo:a}}];class p{}p.\u0275fac=function(n){return new(n||p)},p.\u0275mod=t.oAB({type:p}),p.\u0275inj=t.cJS({imports:[f.Bz.forChild(F),f.Bz]});var y=e(4466);class c{}c.\u0275fac=function(n){return new(n||c)},c.\u0275mod=t.oAB({type:c}),c.\u0275inj=t.cJS({providers:[a],imports:[m.ez,p,y.m]})}}]);