"use strict";(self.webpackChunkblrec=self.webpackChunkblrec||[]).push([[592],{4670:(r,s,c)=>{c.d(s,{g:()=>n});var e=c(4650),i=c(6895);function p(t,o){if(1&t&&(e.TgZ(0,"div",2)(1,"h2",3),e._uU(2),e.qZA()()),2&t){const a=e.oxw();e.xp6(2),e.Oqu(a.name)}}class n{constructor(){this.name=""}}n.\u0275fac=function(o){return new(o||n)},n.\u0275cmp=e.Xpm({type:n,selectors:[["app-page-section"]],inputs:{name:"name"},ngContentSelectors:["*"],decls:3,vars:1,consts:[["class","header",4,"ngIf"],[1,"card"],[1,"header"],[1,"title"]],template:function(o,a){1&o&&(e.F$t(),e.YNc(0,p,3,1,"div",0),e.TgZ(1,"div",1),e.Hsn(2),e.qZA()),2&o&&e.Q6J("ngIf",a.name)},dependencies:[i.O5],styles:["[_nghost-%COMP%]{display:flex;flex-direction:column}.header[_ngcontent-%COMP%]   .title[_ngcontent-%COMP%]{color:#202124;font-size:108%;font-weight:400;letter-spacing:.25px;margin-bottom:1em;margin-top:1.5em}.card[_ngcontent-%COMP%]{flex:1;background-color:#fff;border-radius:4px;box-shadow:0 2px 2px #00000024,0 1px 5px #0000001f,0 3px 1px -2px #0003}"],changeDetection:0})}}]);